/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Lcd_platformservice.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 11/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the LCD platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             11/02/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Macros */

/* USER CODE END Macros */

/* USER CODE BEGIN Global Variables */
#if (SDK_SERVICE_LCD == PS_ENABLE)
extern pvt_variable pv;
extern SDRAM_HandleTypeDef hsdram1;
extern LTDC_HandleTypeDef hltdc;
/* USER CODE END Global Variables */

extern void PS_LTDC_Init(void);
extern void PS_TIM1_Init(void);


/** @brief SET LCD Brightness
 *
 *  This function will set the LCD Brightness value.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
int8_t SET_LCD_Brightness(uint8_t brightness)
{
	uint32_t pulse=0;

	pulse = (((999+1)*brightness)/100);

	TIM1->CCR1 = pulse;

	return PS_SUCCESS;
}

#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief LCD module deinit
 *
 *  This function will make the LCD Enable pin to low.
 *  (The chip enters hardware shutdown mode
 *   when the LCD enabled pin is pulled low)
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void lcd_deinit(void)
{
	/* Set the LCD Brightness value to low */
	SET_LCD_Brightness(MIN_LCD_BRIGHTNESS);

	HAL_TIM_PWM_DeInit(pv.htim1);
	HAL_Delay(10);
	HAL_TIM_Base_DeInit(pv.htim1);
	HAL_Delay(10);
	HAL_LTDC_DeInit(&hltdc);
    HAL_SDRAM_DeInit(&hsdram1);

	/*Configure LCD Enable pin to RESET */
	HAL_GPIO_WritePin(LCD_ENABLE_PORT, LCD_ENABLE_PIN, GPIO_PIN_RESET);
}

/** @brief LCD reinit
 *
 *  This function will make the LCD enable pin to high.
 *  (The chip enters normal mode when the
 *  LCD enable pin is pulled high)
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void lcd_reinit(void)
{
	uint8_t brightness;

	Get_DL(LCD_BRIGHTNESS, &brightness);
	HAL_Delay(10);
	PS_TIM1_Init();
	HAL_Delay(10);
	PS_LTDC_Init();
	HAL_Delay(10);
	/* Set the LCD Brightness value to low */
	SET_LCD_Brightness(brightness);

	/*Configure LCD Enable pin to SET */
	HAL_GPIO_WritePin(LCD_ENABLE_PORT, LCD_ENABLE_PIN, GPIO_PIN_SET);
	HAL_Delay(10);
}
#endif	//SDK_SERVICE_PM


/** @brief LCD DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t Lcd_DB_Init(void)
{
	uint8_t value;

	/* Initialize all the DB variables */
	value = CONF_LCD_STATE;

	if ((LCD_CONF_ON != value) && (LCD_CONF_OFF != value))
		return FAILURE;

	/* Update LCD State from the configuration file */
    Set_DL(LCD_STATE, &value);

    value = CONF_LCD_BRIGHTNESS;

	if(!((MIN_LCD_BRIGHTNESS <= value) &&
			(MAX_LCD_BRIGHTNESS >= value)))
		return FAILURE;

	/* Set the value into the hardware */
	SET_LCD_Brightness(value);

	/* Update LCD Brightness from the configuration file */
    Set_DL(LCD_BRIGHTNESS, &value);

	if(CONF_LCD_STATE == LCD_CONF_OFF)
	{
		HAL_GPIO_WritePin(GPIOE, GPIO_PIN_3, GPIO_PIN_RESET);
		SET_LCD_Brightness(0);
	}
    return PS_SUCCESS;

}

/** @brief LCD Platform service task
 *
 *  This function starts the LCD Platform Service
 *  task.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return none
 *
 */
void Lcd_ServiceStart(void * pvParameters)
{
	uint8_t  *lcdbuf;
	uint32_t cNotifiedValue;
	uint8_t  res = 0;
	uint8_t  func;
	uint8_t brightness_value = 0;

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_LCD_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("LCD Task failed to ping the WD Timer\r\n");
		}
#endif
		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, PS_LCD_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		/* Process the SDK API message */
		if (cNotifiedValue & (1<<LCD_MSG_EVENT))
		{
				/* Receive Message from the LCD SDK API */
				res = xQueueReceive(pv.lcdtxq, &lcdbuf , ( TickType_t ) Q_TICKSTOWAIT );

				if(res != pdTRUE)
				{
					break;
				}

				func = lcdbuf[LCD_FUNC_INDEX];

				/* Update the result */
				lcdbuf[LCD_RESULT_INDEX] = SUCCESS;

				switch(func)
				{
					case SET_LCD:

						switch(lcdbuf[LCD_SUB_FUNC_INDEX])
						{
							case SET_LCD_STATE:

								/* call the HAL API to SET or RESET the LCD pin state */
								if(LCD_CONF_ON == lcdbuf[LCD_STATE_INDEX])
								{
									/* DISP pin enable */
									HAL_GPIO_WritePin(GPIOE, GPIO_PIN_3, GPIO_PIN_SET);

									Get_DL(LCD_BRIGHTNESS, (uint8_t *)&brightness_value);

									/* LCD State ON */
									SET_LCD_Brightness(brightness_value);
								}
								else if(LCD_CONF_OFF == lcdbuf[LCD_STATE_INDEX])
								{
									/* Reset the DISP enable pin */
									HAL_GPIO_WritePin(GPIOE, GPIO_PIN_3, GPIO_PIN_RESET);

									/* LCD State OFF */
									SET_LCD_Brightness(MIN_LCD_BRIGHTNESS);
								}
								break;

							case SET_LCD_BRIGHTNESS:

								/* Call the LCD HAL API to set the brightness */
								SET_LCD_Brightness(lcdbuf[LCD_BRIGHTNESS_INDEX]);
								break;
						}
						break;

					default:
						lcdbuf[LCD_RESULT_INDEX] = FAILURE;
						break;
				}

				/* Send the response to the SDK API */
				xQueueSend( pv.lcdrxq, ( void * )&lcdbuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}

/** @brief Start LCD Platform service
 *
 *  This function initializes the following modules
 *     ===> LCD DB
 *     ===> LCD platform driver
 *     ===> LCD software resources
 *     ===> Start the LCD service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t LCD_PS_Start(void)
{
	BaseType_t xReturned;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(lcd_deinit,NULL,lcd_reinit);
#endif

	/* Create LCD TxQ for receiving value from the SDK API */
	pv.lcdtxq = xQueueCreate( LCD_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.lcdtxq)
	{
		LOGERROR("Unable to create the LCD TxQ \r\n");
		return PS_FAILURE;
	}

	/* Create LCD RxQ for sending return value to the SDK API */
	pv.lcdrxq = xQueueCreate( LCD_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.lcdrxq)
	{
		LOGERROR("Unable to create the LCD RxQ \r\n");
		return PS_FAILURE;
	}

	/* Start the LCD Platform service thread */
	xReturned = xTaskCreate(Lcd_ServiceStart, "LCD",
			512, NULL, PS_LCD_TASK_PRIORITY, &pv.lcd_task);

	if( xReturned != pdPASS )
	{
		LOGERROR("Unable to create the LCD Task \r\n");
		return PS_FAILURE;
	}

	LOGERROR("LCD Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
