/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : LightSensor_platformservice.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Light Sensor platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Macros */


/* USER CODE END Macros */


/* USER CODE BEGIN Global Variables */
#if (SDK_SERVICE_LIGHT_SENSOR == PS_ENABLE)
extern pvt_variable pv;


#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief Light Sensor deinit
 *
 *  This function will make the Light sensor to Shuntdown mode.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void LS_deinit(void)
{
	TakeI2CBusLock();
	/* Shutdown the device */
	SOPTdriver_Shutdown(pv.hi2c);
	GiveI2CBusLock();
}

/** @brief Light Sensor reinit
 *
 *  This function will make the Light sensor to pervious state mode.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void LS_reinit(void)
{
	uint8_t val = 0;
	uint8_t mode = 0;

	if(CONF_TIME_IN_100MS == pv.ls_conv_time)
	{
		val = LS_CONVERSION_TIME_100;
	}
	else
	if(CONF_TIME_IN_800MS == pv.ls_conv_time)
	{
		val = LS_CONVERSION_TIME_800;
	}

	/* Validate the Light Sensor Mode Conversion */
	if ((LS_SINGLE_MODE_CONV != pv.ls_mode) &&
		  (LS_CONTINUOUS_MODE_CONV != pv.ls_mode))
		mode = LS_SINGLE_MODE_CONV;

	TakeI2CBusLock();
    /* Set the conversion time and mode in the device */
	SOPTdriver_Init(pv.hi2c, val, mode);
	GiveI2CBusLock();
}
#endif

#if (SDK_CAM_LS_STOP == PS_ENABLE)
/** @brief Stop/Start Light Sensor Platform service task
 *
 *  This function starts/stops the Light Sensor Platform Service
 *  task.
 *
 *  @param  state : Switch the state of running thread
 *
 *  @return SUCCESS
 *          FAILURE
 *
 */
uint8_t Stop_LS_Service(uint8_t state)
{
	uint8_t *buff;
	uint8_t res;
	/* Get the buffer from the Memory */
   	buff = (uint8_t *) ai430_memory_alloc();


   	/* Check the state */
	if (STATE_STOP == state)
	{
		state = LS_CAM_STOP_EVENT;
	}
	else
		state = LS_CAM_START_EVENT;

    buff[0]  = LS_CAM_STOP_EVENT;

	/* Send the data into the LED Q */
	if (xQueueSend( pv.lstxq, ( void * )&buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS)
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	/* Notify the LS task to update the LS data */
	if (xTaskNotify( pv.ls_task, (1<<state), eSetBits ) != pdPASS)
	{
		return FAILURE;
	}

	/* Wait for receiving the return data */
    /* Send the data to the LS platform service layer */
	if (xQueueReceive(pv.lsrxq, &buff, ( TickType_t ) Q_TICKSTOWAIT ) != pdPASS )
	{
		/* Release the buffer */
		ai430_memory_free(buff);
		return FAILURE;
	}

	res = buff[LS_RESULT_INDEX];

	/* Release the buffer */
	ai430_memory_free(buff);

	return res;
}
#endif

/** @brief Light Sensor DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t LS_DB_Init(void)
{
	uint8_t val;
	uint8_t mode;
	uint16_t mfg;

	TakeI2CBusLock();
	mfg = SOPTdriver_Manufacturer_ID(pv.hi2c);
	GiveI2CBusLock();
	/* Check whether the device is present or not */
	if (LS_MANUFACTURE_ID != mfg)
		return PS_FAILURE;

	val = CONF_LIGHT_SENSOR_CONVERSION_TIME;

	/* Validate the Light Sensor Conversion time */
	if (((LS_CONVERSION_TIME_100 != val) &&
		  (LS_CONVERSION_TIME_800 != val)) || (LS_CONVERSION_TIME_100 == val))
	{
		val = LS_CONVERSION_TIME_100;
		pv.ls_conv_time = CONF_TIME_IN_100MS;
	}
	else
	{
		val = LS_CONVERSION_TIME_800;
		pv.ls_conv_time = CONF_TIME_IN_800MS;
	}

	/* Set the Light Sensor Conversion time in the DB */
	Set_DL(LIGHT_SENSOR_CONVERSION_TIME, &val);

	mode = CONF_LIGHT_SENSOR_CONVERSION_MODE;

	/* Validate the Light Sensor Mode Conversion */
	if ((LS_SINGLE_MODE_CONV != mode) &&
		  (LS_CONTINUOUS_MODE_CONV != mode) && (LS_SHUTDOWN != mode))
		mode = LS_SINGLE_MODE_CONV;

	TakeI2CBusLock();
    /* Set the conversion time and mode in the device */
	SOPTdriver_Init(pv.hi2c, val, mode);
	GiveI2CBusLock();

	pv.ls_mode = mode;

	/* Set the Light Sensor Conversion time in the DB */
	Set_DL(LIGHT_SENSOR_CONVERSION_MODE, &mode);

	return PS_SUCCESS;
}

/** @brief Light Sensor Timer Callback
 *
 *  This function is used for reading the Light sensor data
 *
 *  @param  xTimer : Light Sensor light timer handle
 *
 *  @return none
 *
 */
void LSTimerCallback( TimerHandle_t xTimer )
{
	/* Notify to the WL task to blink the WL */
	xTaskNotify( pv.ls_task, (1<<LS_TMR_EVENT), eSetBits );
}


/** @brief Read Raw LS data
 *
 *  This function reads the Light Sensor data from the device
 *  and update the data to the DB.
 *
 *  @param  None
 *
 *  @return none
 *
 */
static void Update_LightSensor_data(void)
{
	uint32_t fval = 0;
	uint8_t  val = FALSE;
	uint8_t  res;

	/* Timer reaches the converstion time out read the value */
	if (pv.ls_readtime == pv.ls_conv_time)
	{
		TakeI2CBusLock();
		/* Read the data from the device */
		res = SOPTdriver_Result(pv.hi2c, &fval);
		GiveI2CBusLock();

        if (OPTERR_ERROR_OK == res )
        {
			/* Update the data into the DB */
        	Set_DL(LIGHT_SENSOR_DATA, (uint8_t *)&fval);
        }
		if (LS_SINGLE_MODE_CONV == pv.ls_mode)
		{
			/* STOP the timer */
            xTimerStop( pv.xTimers[ PS_LS_TIMER ], 0 );

			/* Update the DB Value */
            Set_DL(LIGHT_SENSOR_TRIGGER, &val);
		}
		pv.ls_readtime = 0;
	}
	else
		pv.ls_readtime ++;
}

/** @brief Light Sensor Platform service task
 *
 *  This function starts the Light Sensor Platform Service
 *  task.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return None
 *
 */
void LS_ServiceStart(void *pvParameters)
{
	uint32_t cNotifiedValue;
	uint32_t lts_val;
	uint8_t *lsbuf;
	uint8_t res;
	uint8_t val;
	uint8_t func;
	TickType_t  th_timeout = PS_LS_TASK_TIMEOUT;
	uint8_t LTS_Trigger = 0;

	/* Register with the Camera Driver */
#if (SDK_CAM_LS_STOP == PS_ENABLE)
	StartStopSDKServiceReg(Stop_LS_Service);
#endif

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(LS_deinit,NULL,LS_reinit);
#endif

	if (pv.ls_mode != LS_SHUTDOWN)
	{
	if ((CONF_TIME_IN_100MS == pv.ls_conv_time) ||
			(CONF_TIME_IN_800MS == pv.ls_conv_time))
	{
		/* Check for the mode to start the timer */
		xTimerStart( pv.xTimers[ PS_LS_TIMER ], 0 );
	}
	} else {
		/* STOP the timer */
        xTimerStop( pv.xTimers[ PS_LS_TIMER ], 0 );
	}

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_LS_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("LS Task failed to ping the WD Timer\r\n");
		}
#endif

		/* Wait for receiving the Event from the SDK API and WL timer */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, th_timeout);

		if(res != pdTRUE)
		{
			continue;
		}

#if (SDK_CAM_LS_STOP == PS_ENABLE)

		/* Camera is streaming, stop the SDK thread */
		if ((cNotifiedValue & (1<< LS_CAM_STOP_EVENT)) &&
				(pv.ls_cam_state != LS_THREAD_STATE_STOP))
		{
			if ((CONF_TIME_IN_100MS == pv.ls_conv_time) ||
					(CONF_TIME_IN_800MS == pv.ls_conv_time))
			{
				/* Check for the mode to start the timer */
				xTimerStop( pv.xTimers[ PS_LS_TIMER ], 0 );
			}

			pv.ls_cam_state = LS_THREAD_STATE_STOP;

		    th_timeout = portMAX_DELAY;
			/* Update the result */
			lsbuf[WL_RESULT_INDEX] = SUCCESS;

			/* Send the response to the SDK API */
			xQueueSend( pv.lsrxq, ( void * )&lsbuf, ( TickType_t ) Q_TICKSTOWAIT );

			continue;
		}

		/* Camera is stopped streaming, start the SDK thread */
		if ((cNotifiedValue & (1<< LS_CAM_START_EVENT)) &&
			(pv.ls_cam_state != LS_THREAD_STATE_RUN))
		{
			if ((CONF_TIME_IN_100MS == pv.ls_conv_time) ||
					(CONF_TIME_IN_800MS == pv.ls_conv_time))
			{
				/* Check for the mode to start the timer */
				xTimerStart( pv.xTimers[ PS_LS_TIMER ], 0 );
			}

			pv.ls_cam_state = WL_THREAD_STATE_RUN;
			th_timeout = PS_LS_TASK_TIMEOUT;

			/* Update the result */
			lsbuf[WL_RESULT_INDEX] = SUCCESS;

			/* Send the response to the SDK API */
			xQueueSend( pv.lsrxq, ( void * )&lsbuf, ( TickType_t ) Q_TICKSTOWAIT );

		    continue;
		}

		/* Receive any event, do not accept. Since the camera is running */
		if (LS_THREAD_STATE_STOP == pv.ls_cam_state)
		{
			/* Update the result */
			lsbuf[WL_RESULT_INDEX] = FAILURE;

			/* Send the response to the SDK API */
			xQueueSend( pv.lsrxq, ( void * )&lsbuf, ( TickType_t ) Q_TICKSTOWAIT );

			continue;
		}
#endif

		/* Event from SDK Light Sensor API */
		Get_DL(LIGHT_SENSOR_TRIGGER, &LTS_Trigger);
		if ((cNotifiedValue & (1<<LS_TMR_EVENT)) || (LTS_Trigger == true))
		{
			/* Check for the WL Blinking */
			Update_LightSensor_data();
		}

		if (cNotifiedValue & (1<<LS_MSG_EVENT))
		{
				/* Receive Message from the Light Sensor SDK API */
				res = xQueueReceive(pv.lstxq, &lsbuf , ( TickType_t ) Q_TICKSTOWAIT );

				if(res != pdTRUE)
				{
					break;
				}

				func = lsbuf[LS_FUNC_INDEX];

				/* Update the result */
				lsbuf[LS_RESULT_INDEX] = SUCCESS;

				switch(func)
				{
					case LS_GET:

						switch(LS_SUB_FUNC_INDEX)
						{
							case LS_GET_CONVERSION_TIME:

								TakeI2CBusLock();
								/* Read Conversion time value from the device */
								res = SOPTdriver_GetCovntime(pv.hi2c, &val);
								GiveI2CBusLock();
								if (OPTERR_ERROR_OK == res)
								{
									lsbuf[LS_RESULT_INDEX+1] = val;
								}
								else
									lsbuf[LS_RESULT_INDEX] = FAILURE;
								break;

							case LS_GET_MODE:
								TakeI2CBusLock();
								/* Read Mode value from the device */
								res = SOPTdriver_GetMode(pv.hi2c, &val);
								GiveI2CBusLock();

								if (OPTERR_ERROR_OK == res)
								{
									lsbuf[LS_RESULT_INDEX + 1] = val;
								}
								else
									lsbuf[LS_RESULT_INDEX] = FAILURE;
								break;

							case LS_READ_DATA:
								TakeI2CBusLock();
								/* Read the RAW data */
								res = SOPTdriver_Result(pv.hi2c, &lts_val);
								GiveI2CBusLock();

								if (OPTERR_ERROR_OK == res)
								{
									/* Update the data into the API buffer */
									memcpy(&lsbuf[LS_RESULT_INDEX+1], &lts_val, sizeof(float));
								}
								else
									lsbuf[LS_RESULT_INDEX] = FAILURE;

							default:
								lsbuf[LS_RESULT_INDEX] = FAILURE;
								break;
						}
						break;

					case LS_SET:

						switch(lsbuf[LS_SUB_FUNC_INDEX])
						{
							case LS_SET_CONVERSION_TIME:

								/* Validate the Conversion Time */
								if (!((LS_CONVERSION_TIME_100 == lsbuf[LS_SUB_FUNC_VAL_INDEX]) ||
										(LS_CONVERSION_TIME_800 == lsbuf[LS_SUB_FUNC_VAL_INDEX])))
								{
									lsbuf[LS_RESULT_INDEX] = FAILURE;
									break;
								}
								TakeI2CBusLock();
								/* Read Conversion time vlaue from the device */
								res = SOPTdriver_GetMode(pv.hi2c, &val);
								GiveI2CBusLock();

								if (OPTERR_ERROR_OK != res)
								{
									lsbuf[LS_RESULT_INDEX] = FAILURE;
								}
								else
								{
									TakeI2CBusLock();
									/* Shutdown the device */
									res = SOPTdriver_Shutdown(pv.hi2c);
									GiveI2CBusLock();
									if (OPTERR_ERROR_OK != res)
									{
										lsbuf[LS_RESULT_INDEX] = FAILURE;
									}
									else
									{
										/* Stop the timer */
										xTimerStop( pv.xTimers[ PS_LS_TIMER ], 0 );
										TakeI2CBusLock();
										/* Restart the Device with new value */
										res = SOPTdriver_Init(pv.hi2c, lsbuf[LS_SUB_FUNC_VAL_INDEX], val);
										GiveI2CBusLock();

										if (OPTERR_ERROR_OK != res)
										{
											lsbuf[LS_RESULT_INDEX] = FAILURE;
										}
										else
										{
											pv.ls_readtime = 0;
											/* Start the timer again */
											xTimerStart( pv.xTimers[ PS_LS_TIMER ], 0 );
										}

										if(LS_CONVERSION_TIME_100 == lsbuf[LS_SUB_FUNC_VAL_INDEX])
										{
											/* Store the conversion time */
											pv.ls_conv_time = CONF_TIME_IN_100MS;
										}

										if(LS_CONVERSION_TIME_800 == lsbuf[LS_SUB_FUNC_VAL_INDEX])
										{
											/* Store the conversion time */
											pv.ls_conv_time = CONF_TIME_IN_800MS;
										}
									}
								}
								break;

							case LS_SET_MODE:

								/* Validate the Mode */
								if (!((LS_SHUTDOWN == lsbuf[LS_SUB_FUNC_VAL_INDEX]) ||
										(LS_SINGLE_MODE_CONV == lsbuf[LS_SUB_FUNC_VAL_INDEX]) ||
										(LS_CONTINUOUS_MODE_CONV == lsbuf[LS_SUB_FUNC_VAL_INDEX])))
								{
									lsbuf[LS_RESULT_INDEX] = FAILURE;
									break;
								}
								TakeI2CBusLock();
								/* Shutdown the device */
								res = SOPTdriver_Shutdown(pv.hi2c);
								GiveI2CBusLock();

								if (OPTERR_ERROR_OK != res)
								{
									lsbuf[LS_RESULT_INDEX] = FAILURE;
								}
								else
								{
									/* LS Device is in Shutdown mode*/
									pv.ls_mode = LS_SHUTDOWN;

									/* Stop the timer */
									xTimerStop( pv.xTimers[ PS_LS_TIMER ], 0 );

									if ((LS_SINGLE_MODE_CONV == lsbuf[LS_SUB_FUNC_VAL_INDEX]) ||
											(LS_CONTINUOUS_MODE_CONV == lsbuf[LS_SUB_FUNC_VAL_INDEX]))
									{
										if (CONF_TIME_IN_100MS == pv.ls_conv_time)
											val = CONF_REG_CONVERSION_TIME_100;
										else
										if (CONF_TIME_IN_800MS == pv.ls_conv_time)
											val = CONF_REG_CONVERSION_TIME_800;
										TakeI2CBusLock();
										/* Restart the Device with new value */
										res = SOPTdriver_Init(pv.hi2c,
												val, lsbuf[LS_SUB_FUNC_VAL_INDEX]);
										GiveI2CBusLock();

										if (OPTERR_ERROR_OK != res)
										{
											lsbuf[LS_RESULT_INDEX] = FAILURE;
										}
										else
										{
											/* LS Device is in Single/Continuous mode*/
											pv.ls_mode = lsbuf[LS_SUB_FUNC_VAL_INDEX];

											/* Start the timer again */
											xTimerStart( pv.xTimers[ PS_LS_TIMER ], 0 );
										}
									}
								}
								break;

							default:
								lsbuf[LS_RESULT_INDEX] = FAILURE;
								break;
						}
						break;

						default:
							/* Update the result */
							lsbuf[LS_RESULT_INDEX] = FAILURE;
							break;
				}

				/* Send the response to the SDK API */
				xQueueSend( pv.lsrxq, ( void * )&lsbuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}

/** @brief Start Light Sensor Platform service
 *
 *  This function initializes the following modules
 *     ===> Light Sensor DB
 *     ===> Light Sensor platform driver
 *     ===> Light Sensor software resources
 *     ===> Start the Light Sensor service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t LS_PS_Start(void)
{

	BaseType_t xReturned;

	/* Create Light Sensor TxQ for receiving value from the SDK API */
	pv.lstxq = xQueueCreate( LS_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.lstxq)
	{
		LOGERROR("Unable to initialize the Light Sensor\r\n");
		return PS_FAILURE;
	}

	/* Create Light Sensor RxQ for sending return value to the SDK API */
	pv.lsrxq = xQueueCreate( LS_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.lsrxq)
	{
		LOGERROR("Unable to initialize the Light Sensor\r\n");
		return PS_FAILURE;
	}

	/* Create the timer for the Reading the Light Sensor value */
	pv.xTimers[ PS_LS_TIMER ] = xTimerCreate
	                   ( "LS_Timer",
	                     PS_LS_RAWREAD_TIMER,
	                     pdTRUE,
	                     ( void * ) 0,
	                     LSTimerCallback
	                   );

	if (NULL == pv.xTimers[ PS_LS_TIMER ])
	{
		LOGERROR("Unable to initialize the LED timer\r\n");
		return PS_FAILURE;
	}
	/* Start the Light Sensor Platform service thread */
	xReturned = xTaskCreate(LS_ServiceStart, "LightSensor",
			512, NULL, PS_LS_TASK_PRIORITY,   &pv.ls_task);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the Light Sensor\r\n");
		return PS_FAILURE;
	}

	LOGERROR("Light Sensor Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
