/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Digitalio.c
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Digital Output HAL driver functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"
#include "../../Drivers/Maximatecc/inc/IO_Expander_TCA9535.h"

/* USER CODE END Includes */

/** @brief DIO Initialize
 *
 *  This function initializes the DIO Expander
 *  IO pins as Output.
 *
 *  @param  None
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t DIO_Init(I2C_HandleTypeDef *hi2c)
{
	HAL_StatusTypeDef ret;
	TCA9535Regs Regs;

	/* Get the IO Expander Semaphore */
	TakeIOExpanderLock();

	/* Read the Config reg from the IO Expander */
	ret = TCA9535ReadConfig(hi2c, &Regs, TCA9535_2_ADDRESS);

	if(HAL_ERROR == ret)
		return PS_FAILURE;

	/* Make the DIO Pins as output */
   	Regs.Config.Port.P0.bit.B0 = 0;
   	Regs.Config.Port.P0.bit.B2 = 0;

	/* Set the DIO IO pins status into the IO Expander */
	ret = TCA9535WriteConfig(hi2c, &Regs, TCA9535_2_ADDRESS);

	/* Read the Polarity Reg from the IO Expander */
	ret = TCA9535ReadPolarity(hi2c, &Regs, TCA9535_2_ADDRESS);

	if(HAL_ERROR == ret)
		return PS_FAILURE;

	/* Change the Polarity Register */
   	Regs.PolarityInversion.Port.P0.bit.B0 = 0;
   	Regs.PolarityInversion.Port.P0.bit.B2 = 0;
	/* Set the DIO IO pins Polarity into the IO Expander */
	ret = TCA9535WritePolarity(hi2c, &Regs, TCA9535_2_ADDRESS);

	/* Release the Semaphore */
	GiveIOExpanderLock();

	return PS_SUCCESS;
}

/** @brief Set Output High
 *
 *  This function sets the Output High in the IO Expander Pins
 *  for the Digital Output High configuration.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */

int8_t Set_OutputHigh(I2C_HandleTypeDef *hi2c)
{
	HAL_StatusTypeDef ret;
	TCA9535Regs Regs;

	/* Get the IO Expander Semaphore */
	TakeIOExpanderLock();

	/* Read the Output Pin from the IO Expander */
	ret = TCA9535ReadOutput(hi2c, &Regs, TCA9535_2_ADDRESS);
	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(20);

	/* OUTPUT
	 * High
	 * L = 0, H = 1
	 **/
	Regs.Output.Port.P0.bit.B0 = 1;/*	CDO_H1 */
	Regs.Output.Port.P0.bit.B2 = 0;/*	CDO_L1 */

	/* Set the LED IO pins status into the IO Expander */
    ret = TCA9535WriteOutput(hi2c, &Regs, TCA9535_2_ADDRESS);

	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(20);

	/* Release the Semaphore */
	GiveIOExpanderLock();

	return PS_SUCCESS;
}



/** @brief Set Output LOW
 *
 *  This function sets the Output Low in the IO Expander Pins
 *  for the Digital Output LOW configuration.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */

int8_t Set_OutputLow(I2C_HandleTypeDef *hi2c)
{
	HAL_StatusTypeDef ret;
	TCA9535Regs Regs;

	/* Get the IO Expander Semaphore */
	TakeIOExpanderLock();

	/* Read the Output Pin from the IO Expander */
	ret = TCA9535ReadOutput(hi2c, &Regs, TCA9535_2_ADDRESS);
	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(100);

	/* OUTPUT
	 * LOW
	 * L = 1, H = 0
	 **/
	Regs.Output.Port.P0.bit.B0 = 0;/*	CDO_H1 */
	Regs.Output.Port.P0.bit.B2 = 1;/*	CDO_L1 */

	/* Set the LED IO pins status into the IO Expander */
    ret = TCA9535WriteOutput(hi2c, &Regs, TCA9535_2_ADDRESS);

	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(100);

	/* Release the Semaphore */
	GiveIOExpanderLock();

	return PS_SUCCESS;
}

/** @brief Set Output Open
 *
 *  This function sets the Output Open in the IO Expander Pins
 *  for the Digital Output Open configuration.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */

int8_t Set_OutputOpen(I2C_HandleTypeDef *hi2c)
{
	HAL_StatusTypeDef ret;
	TCA9535Regs Regs;

	/* Get the IO Expander Semaphore */
	TakeIOExpanderLock();

	/* Read the Output Pin from the IO Expander */
	ret = TCA9535ReadOutput(hi2c, &Regs, TCA9535_2_ADDRESS);
	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(20);

	/* OUTPUT
	 * High
	 * L = 0, H = 0
	 **/
	Regs.Output.Port.P0.bit.B0 = 0;/*	CDO_H1 */
	Regs.Output.Port.P0.bit.B2 = 0;/*	CDO_L1 */

	/* Set the LED IO pins status into the IO Expander */
    ret = TCA9535WriteOutput(hi2c, &Regs, TCA9535_2_ADDRESS);

	if(HAL_ERROR == ret)
		return PS_FAILURE;
	HAL_Delay(20);

	/* Release the Semaphore */
	GiveIOExpanderLock();

	return PS_SUCCESS;
}

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
