/**
 ******************************************************************************
 * @file    is42s32400f.c
 * @author  MCD Application Team
 * @brief   IS42S16100H sdram 8MBx32 driver file
 ******************************************************************************
 * @attention
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "../inc/is42s16100h.h"


/** @addtogroup BSP
  * @{
  */

/** @addtogroup Components
  * @{
  */

/** @defgroup IS42S16100H IS42S16100H
  * @brief     This file provides a set of functions needed to drive the
  *            IS42S16100H SDRAM memory.
  * @{
  */

/** @defgroup IS42S16100H_Private_Variables IS42S16100H Private Variables
  * @{
  */
static FMC_SDRAM_CommandTypeDef Command;
/**
  * @}
  */

/** @defgroup IS42S16100H_Function_Prototypes IS42S16100H Function Prototypes
  * @{
  */
static int32_t IS42S16100H_Delay(uint32_t Delay);

/**
  * @}
  */

/** @defgroup IS42S16100H_Exported_Functions IS42S16100H Exported Functions
  * @{
  */
/**
  * @brief  Initializes the IS42S16100H SDRAm memory
  * @param  Ctx Component object pointer
  * @param  pRegMode : Pointer to Register Mode structure
  * @retval error status
  */
int32_t IS42S16100H_Init(SDRAM_HandleTypeDef *Ctx, IS42S16100H_Context_t *pRegMode)
{
	int32_t ret = IS42S16100H_ERROR;

	/* Step 1: Configure a clock configuration enable command */
	if(IS42S16100H_ClockEnable(Ctx, pRegMode->TargetBank) == IS42S16100H_OK)
	{
		/* Step 2: Insert 100 us minimum delay */
		/* Inserted delay is equal to 1 ms due to systick time base unit (ms) */
		(void)IS42S16100H_Delay(1);

		/* Step 3: Configure a PALL (precharge all) command */
		if(IS42S16100H_Precharge(Ctx, pRegMode->TargetBank) == IS42S16100H_OK)
		{
			(void)IS42S16100H_Delay(1);

			/* Step 4: Configure a Refresh command */
			if(IS42S16100H_RefreshMode(Ctx, pRegMode->TargetBank, pRegMode->RefreshMode) == IS42S16100H_OK)
			{
				/* Step 5: Program the external memory mode register */
				if(IS42S16100H_ModeRegConfig(Ctx, pRegMode) == IS42S16100H_OK)
				{
					/* Step 6: Set the refresh rate counter */
					if(IS42S16100H_RefreshRate(Ctx, pRegMode->RefreshRate) == IS42S16100H_OK)
					{
						ret = IS42S16100H_OK;
					}
				}
			}
		}
	}

	return ret;
}

/**
  * @brief  Enable SDRAM clock
  * @param  Ctx Component object pointer
  * @param  Interface Could be FMC_SDRAM_CMD_TARGET_BANK1 or FMC_SDRAM_CMD_TARGET_BANK2
  * @retval error status
  */
int32_t IS42S16100H_ClockEnable(SDRAM_HandleTypeDef *Ctx, uint32_t Interface)
{
	Command.CommandMode            = IS42S16100H_CLK_ENABLE_CMD;
	Command.CommandTarget          = Interface;
	//Command.AutoRefreshNumber      = 1;
	//Command.ModeRegisterDefinition = 0;

	/* Send the command */
	if(HAL_SDRAM_SendCommand(Ctx, &Command, IS42S16100H_TIMEOUT) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @brief  Precharge all sdram banks
  * @param  Ctx Component object pointer
  * @param  Interface Could be FMC_SDRAM_CMD_TARGET_BANK1 or FMC_SDRAM_CMD_TARGET_BANK2
  * @retval error status
  */
int32_t IS42S16100H_Precharge(SDRAM_HandleTypeDef *Ctx, uint32_t Interface)
{
	Command.CommandMode            = IS42S16100H_PALL_CMD   ;
	Command.CommandTarget          = Interface;
	Command.AutoRefreshNumber      = 1;
	Command.ModeRegisterDefinition = 0;

	/* Send the command */
	if(HAL_SDRAM_SendCommand(Ctx, &Command, IS42S16100H_TIMEOUT) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @brief  Program the external memory mode register
  * @param  Ctx Component object pointer
  * @param  pRegMode : Pointer to Register Mode structure
  * @retval error status
  */
int32_t IS42S16100H_ModeRegConfig(SDRAM_HandleTypeDef *Ctx, IS42S16100H_Context_t *pRegMode)
{
	uint32_t tmpmrd;

	/* Program the external memory mode register */
	tmpmrd = (uint32_t)pRegMode->BurstLength   |\
                       pRegMode->BurstType     |\
                       pRegMode->CASLatency    |\
                       pRegMode->OperationMode |\
                       pRegMode->WriteBurstMode;

	Command.CommandMode            = IS42S16100H_LOAD_MODE_CMD;
	Command.CommandTarget          = pRegMode->TargetBank;
	Command.AutoRefreshNumber      = 1;
	Command.ModeRegisterDefinition = tmpmrd;

	/* Send the command */
	if(HAL_SDRAM_SendCommand(Ctx, &Command, IS42S16100H_TIMEOUT) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @brief  Program the SDRAM timing
  * @param  Ctx Component object pointer
  * @param  pTiming Pointer to SDRAM timing configuration structure
  * @retval error status
  */
int32_t IS42S16100H_TimingConfig(SDRAM_HandleTypeDef *Ctx, FMC_SDRAM_TimingTypeDef *pTiming)
{
	/* Program the SDRAM timing */
	if(HAL_SDRAM_Init(Ctx, pTiming) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @brief  Configure Refresh mode
  * @param  Ctx Component object pointer
  * @param  Interface Could be FMC_SDRAM_CMD_TARGET_BANK1 or FMC_SDRAM_CMD_TARGET_BANK2
  * @param  RefreshMode Could be IS42S16100H_CMD_AUTOREFRESH_MODE or
  *                      IS42S16100H_CMD_SELFREFRESH_MODE
  * @retval error status
  */
int32_t IS42S16100H_RefreshMode(SDRAM_HandleTypeDef *Ctx, uint32_t Interface, uint32_t RefreshMode)
{
	Command.CommandMode            = RefreshMode;
	Command.CommandTarget          = Interface;
	Command.AutoRefreshNumber      = 8;
	Command.ModeRegisterDefinition = 0;

	/* Send the command */
	if(HAL_SDRAM_SendCommand(Ctx, &Command, IS42S16100H_TIMEOUT) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @brief  Set the device refresh rate
  * @param  Ctx Component object pointer
  * @param  RefreshCount The refresh rate to be programmed
  * @retval error status
  */
int32_t IS42S16100H_RefreshRate(SDRAM_HandleTypeDef *Ctx, uint32_t RefreshCount)
{
	/* Set the device refresh rate */
	if(HAL_SDRAM_ProgramRefreshRate(Ctx, RefreshCount) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @brief  Enter Power mode
  * @param  Ctx Component object pointer
  * @param  Interface Could be FMC_SDRAM_CMD_TARGET_BANK1 or FMC_SDRAM_CMD_TARGET_BANK2
  * @retval error status
  */
int32_t IS42S16100H_EnterPowerMode(SDRAM_HandleTypeDef *Ctx, uint32_t Interface)
{
	Command.CommandMode            = IS42S16100H_POWERDOWN_MODE_CMD;
	Command.CommandTarget          = Interface;
	Command.AutoRefreshNumber      = 1;
	Command.ModeRegisterDefinition = 0;

	/* Send the command */
	if(HAL_SDRAM_SendCommand(Ctx, &Command, IS42S16100H_TIMEOUT) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @brief  Exit Power mode
  * @param  Ctx Component object pointer
  * @param  Interface Could be FMC_SDRAM_CMD_TARGET_BANK1 or FMC_SDRAM_CMD_TARGET_BANK2
  * @retval error status
  */
int32_t IS42S16100H_ExitPowerMode(SDRAM_HandleTypeDef *Ctx, uint32_t Interface)
{
	Command.CommandMode            = IS42S16100H_NORMAL_MODE_CMD;
	Command.CommandTarget          = Interface;
	Command.AutoRefreshNumber      = 1;
	Command.ModeRegisterDefinition = 0;

	/* Send the command */
	if(HAL_SDRAM_SendCommand(Ctx, &Command, IS42S16100H_TIMEOUT) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @brief  Sends command to the SDRAM bank.
  * @param  Ctx Component object pointer
  * @param  SdramCmd : Pointer to SDRAM command structure
  * @retval SDRAM status
  */
int32_t IS42S16100H_Sendcmd(SDRAM_HandleTypeDef *Ctx, FMC_SDRAM_CommandTypeDef *SdramCmd)
{
	if(HAL_SDRAM_SendCommand(Ctx, SdramCmd, IS42S16100H_TIMEOUT) != HAL_OK)
	{
		return IS42S16100H_ERROR;
	}
	else
	{
		return IS42S16100H_OK;
	}
}

/**
  * @}
  */

/** @defgroup IS42S16100H_Private_Functions IS42S16100H Private Functions
  * @{
  */

/**
  * @brief This function provides accurate delay (in milliseconds)
  * @param Delay : specifies the delay time length, in milliseconds
  * @retval IS42S16100H_OK
  */
static int32_t IS42S16100H_Delay(uint32_t Delay)
{
//  uint32_t tickstart;
//  tickstart = HAL_GetTick();
//  while((HAL_GetTick() - tickstart) < Delay)
//  {
//  }
	HAL_Delay(Delay);
  return IS42S16100H_OK;
}

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
